/*****************************************************************************
	
	config.hpp --     
	  .

	     Arageli.

	Copyright (C) Nikolai Yu. Zolotykh, 1999--2006
	Copyright (C) Sergey S. Lyalin, 2005--2006
	University of Nizhni Novgorod, Russia

*****************************************************************************/


#ifndef _ARAGELI_config_hpp_
#define _ARAGELI_config_hpp_

/**
	\file
	A lot of configuration switches.
*/

/// Version of the library.
#define ARAGELI_VERSION "2.2.1"


/// Template implementation including switch.
/**    ARAGELI_INCLUDE_CPP_WITH_EXPORT_TEMPLATE,
	 cpp-       
	   include   hpp-,  
	 . */
#define ARAGELI_INCLUDE_CPP_WITH_EXPORT_TEMPLATE


// #define ARAGELI_PLATFORM_OPTIMIZATION_1

// #define ARAGELI_PLATFORM_OPTIMIZATION_2


///	The debug level.
/** Activates assert macros with level-number less then this value.
	Thus, zero deactivates all assert macros and value 3 activates
	all asserts.  This occures only if this macro is not existed already. */

#ifndef ARAGELI_DEBUG_LEVEL
	#define ARAGELI_DEBUG_LEVEL		3
#endif


/// If this macro is defined then assertions throw exception assert_failed.
#define ARAGELI_ASSERT_THROW_EXCEPTION


/// Value of this macro is an integer number that controls the number of iteration in is_prime_small_primes_division function.
#define ARAGELI_IS_PRIME_SMALL_PRIMES_DIVISION_NUMBER 20


#if defined(_MSC_VER) && defined(_WIN32)
	#define ARAGELI_INT64_SUPPORT
#endif


// Please check whether it is correct or not. And only after that uncomment.
//#ifdef __linux
//	#define ARAGELI_LONG_LONG_SUPPORT
//#endif


/// Enables support of signed __int64 and unsigned __int64 types.
//#define ARAGELI_INT64_SUPPORT_ENABLE

/// Enables support of unsigned signed long long and unsigned long long.
//#define ARAGELI_LONG_LONG_SUPPORT_ENABLE

/// Disables support of signed __int64 and unsigned __int64 types.
//#define ARAGELI_INT64_SUPPORT_DISABLE

/// Disables support of unsigned signed long long and unsigned long long.
//#define ARAGELI_LONG_LONG_SUPPORT_DISABLE

#if defined(ARAGELI_INT64_SUPPORT_ENABLE) && defined(ARAGELI_INT64_SUPPORT_DISABLE)
	#error Both macros ARAGELI_INT64_SUPPORT_DISABLE and \
		ARAGELI_INT64_SUPPORT_ENABLE are defined.
#endif


#if defined(ARAGELI_LONG_LONG_SUPPORT_ENABLE) && defined(ARAGELI_LONG_LONG_SUPPORT_DISABLE)
	#error Both macros ARAGELI_LONG_LONG_SUPPORT_DISABLE and \
		ARAGELI_LONG_LONG_SUPPORT_ENABLE are defined.
#endif


#ifdef ARAGELI_INT64_SUPPORT_ENABLE
	#define ARAGELI_INT64_SUPPORT
#elif defined(ARAGELI_INT64_SUPPORT_DISABLE) && defined(ARAGELI_INT64_SUPPORT)
	#undef ARAGELI_INT64_SUPPORT
#endif


#ifdef ARAGELI_LONG_LONG_SUPPORT_ENABLE
	#define ARAGELI_LONG_LONG_SUPPORT
#elif defined(ARAGELI_LONG_LONG_SUPPORT_DISABLE) && defined(ARAGELI_LONG_LONG_SUPPORT)
	#undef ARAGELI_LONG_LONG_SUPPORT
#endif


#define ARAGELI_DISABLE_PARTICULAR_COMPILER_WARNINGS


#define ARAGELI_USE_AUTOLINK

#ifdef ARAGELI_USE_AUTOLINK_ENABLE
	#define ARAGELI_USE_AUTOLINK
#elif defined(ARAGELI_USE_AUTOLINK_DISABLE) && defined(ARAGELI_USE_AUTOLINK)
	#undef ARAGELI_USE_AUTOLINK
#endif


#ifdef ARAGELI_USE_AUTOLINK
	#define ARAGELI_LIB_NAME arageli
	#include "auto_link.hpp"
#endif


#endif  //  #ifndef _ARAGELI_config_hpp_
