// Author : Aleksandrov Vladimir
// Tester : Sidnev Alexey 
// Creation date : 20.01.2006
// Modification date: 30.01.2006
// Testing date: 12.02.2006
// Modified by Sergey S. Lyalin, 2006-08-31
// Description :

//1)  D1
//--  

#include "../ts_stdafx.hpp"
#include "rand.hpp"

#include <time.h>
#include <strstream>

using namespace Arageli;

template <class T>
bool sparse_polynom_multiply_divide_monom (const sparse_polynom<T> A, const T B, const int C)
{
	try
	{	
		typedef typename sparse_polynom<T>::monom monom;
		
		sparse_polynom<T> a=A, x=A;
		a *= monom(T (B),C);
		a /= monom(T (B),C);
	
		if (x!=a)
		{
			tout<<"function multiply_divide_scalar failed on 1 check with A="<<A<<", B="<<B<<", C="<<C<<'\n';
			return true;
		}

		if ((a!=(a/monom(T (B),C))*monom(T (B),C)+a%monom(T (B),C)))
		{
			tout<<"function multiply_divide_scalar failed on 2 check with A="<<A<<", B="<<B<<", C="<<C<<'\n';
			return true;
		}
		
		return false;
	}
	catch(Arageli::exception& e)
	{
		tout << "EXCEPTION: ";
		e.output(tout); tout << '\n';
		return true;
	}

}

bool sparse_polynom_multiply_divide_monom_test(int param, int count)
{
	bool fail=false;
	RNG gen(param,1<<16);

		sparse_polynom<int> c = "0";
		fail |= sparse_polynom_multiply_divide_monom(c,1,0);

		if(fail) 
		{
			tout<<"Function sparse_polynom_multiply_divide_monom_test failed on first step.\n";
			return fail;
		}

	
		for (int k=0; k<count; k++)
		{
			int a=rand();
			int b=rand();
			sparse_polynom<int> S = "2*x^2+5*x-7+3*x";
			sparse_polynom<big_int> B = "1234567891011121314151617181920*x^777-9876543210*x^123+112233445566778899";
			sparse_polynom<double> D = "1.12345*x-1e45*x^2+1234.5678*x^3-0.000002334";
			sparse_polynom<rational<> > R = "1234/56781*x^321+7/9*x+3*x^2-4/256";
			fail |= sparse_polynom_multiply_divide_monom(S,a,b);
			fail |= sparse_polynom_multiply_divide_monom(B,big_int (a),b);
//		1	fail |= sparse_polynom_multiply_divide_monom(D,D1);
			fail |= sparse_polynom_multiply_divide_monom(R,rational<>(a,2),b);

			if(fail) 
			{
				tout<<"Function sparse_polynom_multiply_divide_monom_test failed on "<<k+1<<" step.\n";
				return fail;
			}
		}
	
	return false;
}




TEST(sparse_polynom,multiply_divide_monom,"Test")
{ 
/*	
	sparse_polynom<big_int> sp("2*x^2+5*x-7+3*x");
	monom<big_int> m(41,3456);
	tout << sp<<"  "<<m <<"  "<<sp/m<<'\n';
*/
	bool fail=sparse_polynom_multiply_divide_monom_test(100,10);
	
	if(fail) return resFAIL;
	return resOK;

}
